@@ -54,7 +54,7 @@
 #else
 #   define LOG_MUTEX(expr)  PJ_LOG(6,expr)
 #endif
-
+#   define LOG_MUTEX_WARN(expr)  PJ_PERROR(3,expr)
 #define THIS_FILE       "os_core_win32.c"
 
 /*
@@ -480,7 +480,8 @@ static void set_thread_display_name(const char *name)
     PJ_LOG(5, (THIS_FILE, "SetThreadDescription:%p, name:%s", fn, name));
     if (fn) {
         wchar_t wname[PJ_MAX_OBJ_NAME];
-        pj_ansi_to_unicode(name, pj_ansi_strlen(name), wname, PJ_MAX_OBJ_NAME);
+        pj_ansi_to_unicode(name, (int)pj_ansi_strlen(name), wname,
+                           PJ_MAX_OBJ_NAME);
         fn(GetCurrentThread(), wname);
         return;
     }
@@ -1094,10 +1095,15 @@ PJ_DEF(pj_status_t) pj_mutex_lock(pj_mutex_t *mutex)
         status = PJ_STATUS_FROM_OS(GetLastError());
 
 #endif
-    LOG_MUTEX((mutex->obj_name, 
-              (status==PJ_SUCCESS ? "Mutex acquired by thread %s" : "FAILED by %s"),
-              pj_thread_this()->obj_name));
-
+    if (status == PJ_SUCCESS) {
+        LOG_MUTEX((mutex->obj_name, 
+                   "Mutex acquired by thread %s",
+                   pj_thread_this()->obj_name));
+    } else {
+        LOG_MUTEX_WARN((mutex->obj_name, status,
+                        "Failed to acquire mutex by thread %s",
+                        pj_thread_this()->obj_name));
+    }
 #if PJ_DEBUG
     if (status == PJ_SUCCESS) {
         mutex->owner = pj_thread_this();
@@ -1279,6 +1285,7 @@ PJ_DEF(pj_status_t) pj_sem_create( pj_pool_t *pool,
 static pj_status_t pj_sem_wait_for(pj_sem_t *sem, unsigned timeout)
 {
     DWORD result;
+    pj_status_t status = PJ_SUCCESS;
 
     PJ_CHECK_STACK();
     PJ_ASSERT_RETURN(sem, PJ_EINVAL);
@@ -1296,16 +1303,15 @@ static pj_status_t pj_sem_wait_for(pj_sem_t *sem, unsigned timeout)
         LOG_MUTEX((sem->obj_name, "Semaphore acquired by thread %s", 
                                   pj_thread_this()->obj_name));
     } else {
-        LOG_MUTEX((sem->obj_name, "Semaphore: thread %s FAILED to acquire", 
-                                  pj_thread_this()->obj_name));
+        if (result == WAIT_TIMEOUT)
+            status = PJ_ETIMEDOUT;
+        else
+            status = PJ_RETURN_OS_ERROR(GetLastError());
+        LOG_MUTEX_WARN((sem->obj_name, status, "Semaphore: thread %s failed "
+                        "to acquire", pj_thread_this()->obj_name));
     }
 
-    if (result==WAIT_OBJECT_0)
-        return PJ_SUCCESS;
-    else if (result==WAIT_TIMEOUT)
-        return PJ_ETIMEDOUT;
-    else
-        return PJ_RETURN_OS_ERROR(GetLastError());
+    return status;
 }
 
 /*
