@@ -29,6 +29,12 @@
     #define COREAUDIO_MAC 1
 #endif
 
+#if (TARGET_OS_OSX && defined(__MAC_12_0))
+    #define AUDIO_OBJECT_ELEMENT_MAIN kAudioObjectPropertyElementMain;
+#else
+    #define AUDIO_OBJECT_ELEMENT_MAIN kAudioObjectPropertyElementMaster;
+#endif
+
 #include <AudioUnit/AudioUnit.h>
 #include <AudioToolbox/AudioConverter.h>
 #if COREAUDIO_MAC
@@ -447,7 +453,7 @@ static pj_status_t ca_factory_refresh(pjmedia_aud_dev_factory *f)
     /* Find out how many audio devices there are */
     addr.mSelector = kAudioHardwarePropertyDevices;
     addr.mScope = kAudioObjectPropertyScopeGlobal;
-    addr.mElement = kAudioObjectPropertyElementMaster;
+    addr.mElement = AUDIO_OBJECT_ELEMENT_MAIN;
     ostatus = AudioObjectGetPropertyDataSize(kAudioObjectSystemObject, &addr,
                                              0, NULL, &dev_size);
     if (ostatus != noErr) {
@@ -492,7 +498,7 @@ static pj_status_t ca_factory_refresh(pjmedia_aud_dev_factory *f)
         /* Find default audio input device */
         addr.mSelector = kAudioHardwarePropertyDefaultInputDevice;
         addr.mScope = kAudioObjectPropertyScopeGlobal;
-        addr.mElement = kAudioObjectPropertyElementMaster;
+        addr.mElement = AUDIO_OBJECT_ELEMENT_MAIN;
         size = sizeof(dev_id);
         
         ostatus = AudioObjectGetPropertyData(kAudioObjectSystemObject,
@@ -544,7 +550,7 @@ static pj_status_t ca_factory_refresh(pjmedia_aud_dev_factory *f)
         /* Get device name */
         addr.mSelector = kAudioDevicePropertyDeviceName;
         addr.mScope = kAudioObjectPropertyScopeGlobal;
-        addr.mElement = kAudioObjectPropertyElementMaster;
+        addr.mElement = AUDIO_OBJECT_ELEMENT_MAIN;
         size = sizeof(cdi->info.name);
         AudioObjectGetPropertyData(cdi->dev_id, &addr,
                                    0, NULL,
