@@ -281,8 +281,9 @@ static void reset_dev_info(struct avi_dev_info *adi)
     pj_bzero(adi, sizeof(*adi));
 
     /* Fill up with *dummy" device info */
-    pj_ansi_strncpy(adi->info.name, "AVI Player", sizeof(adi->info.name)-1);
-    pj_ansi_strncpy(adi->info.driver, DRIVER_NAME, sizeof(adi->info.driver)-1);
+    pj_ansi_strxcpy(adi->info.name, "AVI Player", sizeof(adi->info.name));
+    pj_ansi_strxcpy(adi->info.driver, DRIVER_NAME, 
+                    sizeof(adi->info.driver));
     adi->info.dir = PJMEDIA_DIR_CAPTURE;
     adi->info.has_callback = PJ_FALSE;
 }
@@ -397,7 +398,10 @@ PJ_DEF(pj_status_t) pjmedia_avi_dev_alloc( pjmedia_vid_dev_factory *f,
     /* Reinit */
     PJ_ASSERT_RETURN(p->path.slen, PJ_EINVAL);
     adi->pool = pj_pool_create(cf->pf, "avidi%p", 512, 512, NULL);
-
+    if (!adi->pool) {
+        status = PJ_ENOMEM;
+        goto on_error;
+    }
 
     /* Open the AVI */
     pj_strdup_with_null(adi->pool, &adi->fpath, &p->path);
@@ -484,8 +488,10 @@ PJ_DEF(pj_status_t) pjmedia_avi_dev_alloc( pjmedia_vid_dev_factory *f,
     }
 
     /* Init device info */
-    pj_ansi_strncpy(adi->info.name, adi->title.ptr, sizeof(adi->info.name)-1);
-    pj_ansi_strncpy(adi->info.driver, DRIVER_NAME, sizeof(adi->info.driver)-1);
+    pj_ansi_strxcpy(adi->info.name, adi->title.ptr, 
+                    sizeof(adi->info.name));
+    pj_ansi_strxcpy(adi->info.driver, DRIVER_NAME, 
+                    sizeof(adi->info.driver));
     adi->info.dir = PJMEDIA_DIR_CAPTURE;
     adi->info.has_callback = PJ_FALSE;
 
