@@ -236,6 +236,8 @@ static void keystroke_help()
     puts("|  a  Answer call              |  i  Send IM              | !a  Modify accnt. |");
     puts("|  h  Hangup call  (ha=all)    |  s  Subscribe presence   | rr  (Re-)register |");
     puts("|  H  Hold call                |  u  Unsubscribe presence | ru  Unregister    |");
+    puts("|  o Toggle call SDP offer     |  D  Subscribe dlg event  |                   |");
+    puts("|                              |  Du Unsub dlg event      |                   |");
     puts("|  v  re-inVite (release hold) |  t  Toggle online status |  >  Cycle next ac.|");
     puts("|  U  send UPDATE              |  T  Set online status    |  <  Cycle prev ac.|");
     puts("| ],[ Select next/prev call    +--------------------------+-------------------+");
@@ -732,6 +734,9 @@ static void ui_make_new_call()
 
         pjsua_msg_data_init(&msg_data_);
         TEST_MULTIPART(&msg_data_);
+        if (app_config.enable_loam) {
+            call_opt.flag |= PJSUA_CALL_NO_SDP_OFFER;
+        }
         pjsua_call_make_call(current_acc, &tmp, &call_opt, NULL,
                              &msg_data_, &current_call);
 
@@ -771,6 +776,10 @@ static void ui_make_multi_call()
         tmp = pj_str(result.uri_result);
     }
 
+    if (app_config.enable_loam) {
+        call_opt.flag |= PJSUA_CALL_NO_SDP_OFFER;
+    }
+
     for (i=0; i<my_atoi(menuin); ++i) {
         pj_status_t status;
 
@@ -822,26 +831,30 @@ static void ui_send_instant_message()
 
 
     /* Send typing indication. */
-    if (i != -1)
-        pjsua_call_send_typing_ind(i, PJ_TRUE, NULL);
-    else {
-        pj_str_t tmp_uri = pj_str(uri);
-        pjsua_im_typing(current_acc, &tmp_uri, PJ_TRUE, NULL);
+    if (!app_config.no_mci) {
+        if (i != -1)
+            pjsua_call_send_typing_ind(i, PJ_TRUE, NULL);
+        else {
+            pj_str_t tmp_uri = pj_str(uri);
+            pjsua_im_typing(current_acc, &tmp_uri, PJ_TRUE, NULL);
+        }
     }
 
     /* Input the IM . */
     if (!simple_input("Message", text, sizeof(text))) {
-        /*
-        * Cancelled.
-        * Send typing notification too, saying we're not typing.
-        */
-        if (i != -1)
-            pjsua_call_send_typing_ind(i, PJ_FALSE, NULL);
-        else {
-            pj_str_t tmp_uri = pj_str(uri);
-            pjsua_im_typing(current_acc, &tmp_uri, PJ_FALSE, NULL);
+        if (!app_config.no_mci) {
+            /*
+            * Cancelled.
+            * Send typing notification too, saying we're not typing.
+            */
+            if (i != -1)
+                pjsua_call_send_typing_ind(i, PJ_FALSE, NULL);
+            else {
+                pj_str_t tmp_uri = pj_str(uri);
+                pjsua_im_typing(current_acc, &tmp_uri, PJ_FALSE, NULL);
+            }
+            return;
         }
-        return;
     }
 
     tmp = pj_str(text);
@@ -991,7 +1004,10 @@ static void ui_add_buddy()
     pj_bzero(&buddy_cfg, sizeof(pjsua_buddy_config));
 
     buddy_cfg.uri = pj_str(buf);
-    buddy_cfg.subscribe = PJ_TRUE;
+    /* Only one subscription can be active, so we need to disable this
+     * to allow user to choose between presence or dialog event.
+     */
+    // buddy_cfg.subscribe = PJ_TRUE;
 
     status = pjsua_buddy_add(&buddy_cfg, &buddy_id);
     if (status == PJ_SUCCESS) {
@@ -1072,6 +1088,11 @@ static void ui_delete_account()
     }
 }
 
+static void ui_unset_loam_mode()
+{
+    app_config.enable_loam = PJ_FALSE;
+}
+
 static void ui_call_hold()
 {
     if (current_call != -1) {
@@ -1090,6 +1111,9 @@ static void ui_call_reinvite()
 static void ui_send_update()
 {
     if (current_call != -1) {
+        if (app_config.enable_loam) {
+            call_opt.flag |= PJSUA_CALL_NO_SDP_OFFER;
+        }
         pjsua_call_update2(current_call, &call_opt, NULL);
     } else {
         PJ_LOG(3,(THIS_FILE, "No current call"));
@@ -1434,6 +1458,18 @@ static void ui_send_arbitrary_request()
     }
 }
 
+static void ui_toggle_call_sdp_offer()
+{
+    app_config.enable_loam = !app_config.enable_loam;
+
+    printf("Subsequent calls and UPDATEs will contain SDP offer: ");
+    if (app_config.enable_loam) {
+        printf("NO.\n");
+    } else {
+        printf("YES.\n");
+    }
+}
+
 static void ui_echo(char menuin[])
 {
     if (pj_ansi_strnicmp(menuin, "echo", 4)==0) {
@@ -1496,6 +1532,32 @@ static void ui_subscribe(char menuin[])
     }
 }
 
+static void ui_subscribe_dlg_event(pj_bool_t sub)
+{
+    char buf[128];
+    input_result result;
+
+    ui_input_url("(un)Subscribe dialog event of", buf, sizeof(buf), &result,
+                 PJ_TRUE);
+    if (result.nb_result != PJSUA_APP_NO_NB) {
+        if (result.nb_result == -1) {
+            int i, count;
+            count = pjsua_get_buddy_count();
+            for (i=0; i<count; ++i)
+                pjsua_buddy_subscribe_dlg_event(i, sub);
+        } else if (result.nb_result == 0) {
+            puts("Sorry, can only subscribe to buddy's dialog event, "
+                 "not from existing call");
+        } else {
+            pjsua_buddy_subscribe_dlg_event(result.nb_result-1, sub);
+        }
+
+    } else if (result.uri_result) {
+        puts("Sorry, can only subscribe to buddy's dialog event, "
+             "not arbitrary URL (for now)");
+    }
+}
+
 static void ui_register(char menuin[])
 {
     switch (menuin[1]) {
@@ -1786,8 +1848,6 @@ static void ui_handle_ip_change()
     status = pjsua_handle_ip_change(&param);
     if (status != PJ_SUCCESS) {
         pjsua_perror(THIS_FILE, "IP change failed", status);
-    } else {
-        PJ_LOG(3,(THIS_FILE, "IP change succeeded"));
     }
 }
 
@@ -1907,6 +1967,13 @@ void legacy_main(void)
             ui_call_hold();
             break;
 
+        case 'o':
+            /*
+             * Toggle call SDP offer
+             */
+            ui_toggle_call_sdp_offer();
+            break;
+
         case 'v':
 #if PJSUA_HAS_VIDEO
             if (menuin[1]=='i' && menuin[2]=='d' && menuin[3]==' ') {
@@ -1987,6 +2054,11 @@ void legacy_main(void)
             ui_subscribe(menuin);
             break;
 
+        case 'D':
+            /* Subscribe/unsubscribe dialog event */
+            ui_subscribe_dlg_event(menuin[1] != 'u');
+            break;
+
         case 'r':
             ui_register(menuin);
             break;
