@@ -937,6 +937,8 @@ PJ_DEF(pj_status_t) pjsip_find_msg( const char *buf, pj_size_t size,
 
     /* Found Content-Length? */
     if (content_length == -1) {
+        /* As per RFC3261 7.4.2, 7.5, 20.14, Content-Length is mandatory over TCP. */
+        PJ_LOG(2, (THIS_FILE, "Field Content-Length not found!"));
         return status;
     }
 
@@ -1529,7 +1531,17 @@ static void* int_parse_sip_url( pj_scanner *scanner,
     }
 
     if (int_is_next_user(scanner)) {
+#if defined (PJSIP_URI_USE_ORIG_USERPASS) && (PJSIP_URI_USE_ORIG_USERPASS)
+        char *start = scanner->curptr;
+        pj_str_t orig;
+#endif
+
         int_parse_user_pass(scanner, pool, &url->user, &url->passwd);
+
+#if defined (PJSIP_URI_USE_ORIG_USERPASS) && (PJSIP_URI_USE_ORIG_USERPASS)
+        pj_strset3(&orig, start, scanner->curptr - 1);
+        pj_strdup(pool, &url->orig_userpass, &orig);
+#endif
     }
 
     /* Get host:port */
