@@ -1,13 +1,12 @@
-# $Id$
-import imp
 import sys
 import inc_sip as sip
 import inc_const as const
+import inc_util as util
 import re
 from inc_cfg import *
 
 # Read configuration
-cfg_file = imp.load_source("cfg_file", ARGS[1])
+cfg_file = util.load_module_from_file("cfg_file", ARGS[1])
 
 # Test body function
 def test_func(t):
